#include "InOut.h"
#include "monitor.h"
#include "main.h"

uint8_t disp1,disp2;		// contents of OUT Ports 1,2
uint8_t keyRow,keyCol;
uint8_t raw1[8], raw2[8];		// unsorted display contents
uint8_t counter = 0;			// counter for display service

// keyboard layout translation from Wincor Nixdorf based layout to original K1001
const uint16_t keymap[] = {
0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,
0x0702,0x0702,0x0802,0x0802,0x0902,0x0902,0x0A02,0x0A02,0x0000,0x0C02,0x0D02,0x0E02,
0x0000,0x0104,0x0000,0x0000,0x0000,0x0000,0x0B80,0x0B20,0x0B08,0x0C04,0x0D04,0x0E04,
0x0008,0x0108,0x0208,0x0508,0x0608,0x0708,0x0808,0x0908,0x0A08,0x0C08,0x0D08,0x0E08,
0x0010,0x0110,0x0210,0x0510,0x0610,0x0710,0x0810,0x0910,0x0A10,0x0C10,0x0D10,0x0E10,
0x0020,0x0120,0x0220,0x0520,0x0620,0x0720,0x0820,0x0920,0x0A20,0x0C20,0x0D20,0x0E20,
0x0040,0x0140,0x0240,0x0540,0x0640,0x0740,0x0840,0x0940,0x0A40,0x0C40,0x0D40,0x0E40,
0x0080,0x0180,0x0280,0x0580,0x0680,0x0780,0x0880,0x0980,0x0A80,0x0C80,0x0D80,0x0E80	};

// as *PRLIMINARY* solution unused IN instructions set A register to 0

void in_0()
{
	if (scanKeyb) {		// in scan mode {1,2,3,4...}
		Areg = 0;		// first assume nothing found...
		if (scanKeyb == (keyCol+1)) {
			 Areg = keyRow;
			 scanKeyb = 0;	// end scan mode if key found
		}	
		else scanKeyb++;
		if (scanKeyb>15) {
			scanKeyb = 0;	// end scan mode if all cols checked
		}		
	}
	else Areg = ColRow & 0xFF;	// generic check for recent keypress
	PC++;
}

void in_1()
{
	Areg = 0;
	PC++;
}

void in_2()		// Ausrstungs-Variante des K1000
{
	Areg = 7;		// K1003
//	Areg = 3;		K1002 mit Magnetkarten-Einheit (MKE)
//	Areg = 0;		nur Basisgert
	PC++;
}

void in_3()
{
	Areg = 0;
	PC++;
}

void in_4()
{
	Areg = 4;		// Drucker immer sofort bereit
	PC++;
}

void in_5()
{
	Areg = 0;
	PC++;
}

void in_6()
{
	Areg = 0;
	PC++;
}

void in_7()
{
	Areg = 0;
	PC++;
}
// -----------------

void out_0()
{
uint8_t i,bit;
/*
// **** TEST SECTION ***
static uint8_t count,prev,r,c;
uint8_t x;
	PORTA = 0;			// erase previous number
	x = 1;
	if (counter) x=0;
	PORTB = x;			// 1 every 16. time, else 0
	PORTB |= 0b010;
	PORTB &= 0b000;		// impuls 0-1-0 for SCK on PB1
	PORTB |= 0b100;
	PORTB &= 0b000;		// impuls 0-1-0 for RCK on PB2, also switch off PB0
	if (sp_key) dispDelay = 7;		// keep Display alive while in Special mode
	if (dispDelay) {
		dispDelay--;	// count down to 0, then leave at 0
		PORTA = dispBuffer[counter]; // display current char, only if dispDelay > 0
	}	
	if ((x = PINC & 0b11111110)) {	// this is an assignment combined with a test!
		r = x; c = counter;
	}	
	counter++;
	if (counter > 15) {
		if (r == prev) count++;
		else count = 0;		
		prev = r;
		if (count > 1) {
			keydown = r;			// keydown is a flag, 0=FALSE, <any other value>=TRUE
			if (r==4 && c==1) sp_key=9;	// Serial Monitor
			if (r==2 && c>9) {		// extra treatment for keys MKL, MKS
				sp_key = c;			// set flag for special key request (10 | 11)
			} 
			else ColRow = (c<<8) | r;
		}		
		r = c = counter = 0;
	}	
// **** END OF TEST SECTION ***
*/
//	printf("Addr %d , OUT 0, %d\n",PC,Areg);
	//collect display bits
	bit = 1;	
	for (i=0;i<8;i++) {
		if (disp1 & bit) raw1[i] |= Areg;		
		if (disp2 & bit) raw2[i] |= Areg;
		bit *= 2;
	}

	// if Areg = 128 then facilitate translation, output and clear raw display registers for next round
	if (Areg == 128) {
		for (i=0;i<8;i++) {
			dispBuffer[i] = raw1[i];
			dispBuffer[i+8] = raw2[i];
			raw1[i] = raw2[i] = 0;
		}		
		/* TESTCODE
		dispBuffer[0] = ColRow & 0xFF;  // row code
		dispBuffer[1] = ColRow >> 8;	// col code
		END TESTCODE */
	}
	PC++;
}

void out_1()
{
	disp1 = Areg;	// remember it for subsequent OUT 0
	PC++;
}

void out_2()
{
	disp2 = Areg;	// remember it for subsequent OUT 0
	PC++;
}

// OUT 3 sets the K1000 status LEDs
void out_3()
{
	PORTD = (uint8_t) ((Areg << 2) & 0b11011100);
	PC++;
}

void out_4()
{
	PC++;
}

void out_5()
{
	PC++;
}

void out_6()
{
	PC++;
}

void out_7()
{
	PC++;
}

void out_8()	// disable display(?)
{
	dispDelay = 2;
	PC++;
}

void out_9()
{
	if (Areg == 254) {
		scanKeyb = 1;	// OUT 0,254 initiates a key scan
		getKey();		// into global vars keyRow, keyCol
	}	
	PC++;
}

void out_10()
{
	PC++;
}

void out_11()	// enable display(?)
{
	PC++;
}

void out_12()
{
	PC++;
}

void out_13()
{
	PC++;
}

void out_14()
{

	PC++;
}

void out_15()
{

	PC++;
}

void out_16()
{

	PC++;
}

void out_17()
{

	PC++;
}

void out_18()
{

	PC++;
}

void out_19()
{

	PC++;
}

void out_20()
{

	PC++;
}

void out_21()
{

	PC++;
}

void out_22()
{

	PC++;
}

void out_23()
{
	if (Areg == 0) PrintLine();
	PC++;
}

void getKey()
{
uint16_t x;
	keyRow = ColRow & 0xFF;
	keyCol = ColRow >> 8;
	// transformation of keyboard layout
	x = 0;
	while (!(keyRow & 1)) {
		x++; keyRow = keyRow >> 1;
	}
	x *= 12;
	x += keyCol;
	keyCol = keymap[x] >> 8;
	keyRow = keymap[x] & 0xFF;
	// end of keyboard layout transformation
}
