#include "main.h"
#include "SDcard.h"
#include "mmc.h"	// SD-Card Support (based on Ulrich Radigs code)
#include <avr/interrupt.h>
#include <util/delay.h>

// read a K1000 program from SD-Card
void ReadCard(void)
{
uint16_t blkno, i=0, j;
uint16_t addr;
uint8_t b;
uint8_t checksum;
	blkno = getArg();	// get block number for SD-Card I/O
	blkno = blkno << 3;	// multiply * 8 (* 512 Byte Blocksize) to have 4K per Slot
	addr = KRam[0x80A] + KRam[0x80B] * 256;		// current Program Counter
	cli();	// disable interrupts
	if (mmc_init()) hlt();						// stop if SD-Card init fails
	do {
		mmc_read_sector((uint32_t)(blkno + i), KRam);
		checksum = 0;	// reset checksum for each block
		for (j=1;j<512;j++) {
			checksum += (KRam[j] + 1);
			KRam[addr++] = KRam[j];
			if ((KRam[j] == 0x75) || (addr == RAMTOP)) break;		// 0x75 is 'End Of Program' code
		}
		if (checksum != KRam[0]) hlt();		// error break in case checksum does not match
		if ((KRam[j] == 0x75) || (addr == RAMTOP)) break;
		i++;		// next block on SD-Card
	} while (i<8);	// should never happen	
	sei();	// allow interrupts again
	addr--;	// adjust addr to End Of Program or End Of RAM
	// finally manipulate new program counter into K1000 status variables
	KRam[0x80A] = (uint8_t)(addr & 0xFF);			// put absolute, bin. PC to 0x80A
	KRam[0x80B] = (uint8_t)(addr >> 8);
	i = addr - (KRam[0x806] + 256 * KRam[0x807]);	// calculate relative PC
    b = i % 10; i /= 10; KRam[0x808] = b;			// convert in BCD format
	b = i % 10; i /= 10; KRam[0x808] |= (b << 4);
    b = i % 10; i /= 10; KRam[0x809] = b;
	b = i;KRam[0x809] |= (b << 4);
}

// write a K1000 program to SD-Card
void WriteCard(void)
{
uint16_t blkno,i=0,j;
uint16_t addr;
uint8_t stat = 1;	
	blkno = getArg();
	blkno = blkno << 3;
	if (CheckBlock(blkno)) {						// if 1st Block contains something <> 0
		stat = Confirm();							
	}												// ask for confirmation to overwrite
	if (stat) {										// overwrite ONLY if "L" was pressed											
		addr = KRam[0x80A] + KRam[0x80B] * 256;		// current Program Counter
		cli();	// disable interrupts
		if (mmc_init()) hlt();						// stop if SD-Cards init fails	
		do {
			KRam[0] = 0;	// reset checksum for each new block
			for (j=1;j<512;j++) {					// (1 Byte Checksum, 511 Byte Data) per block
				if (addr < RAMTOP) {
					KRam[0] += (KRam[addr] + 1);		// 1st Byte is checksum
					KRam[j] = KRam[addr++];
					if (KRam[j] == 0x75) break;		// 0x75 is 'End Of Program' code
				}
				else {
					KRam[0]++;		// above RAMTOP just put ZEROs as dummy content
					KRam[j] = 0;				
				}			
			}
			mmc_write_sector((uint32_t)(blkno + i), KRam);
			if (KRam[j] == 0x75) break;
			i++;		// next block on SD-Card
		} while (i<8);	// should never happen	
		sei();	// allow interrupts again
	}	
}

uint16_t getArg(void)  // get 3 digit decimal input for MKL, MKS
{
uint16_t Arg = 0;
uint8_t i = 0;
	PORTD |= 0b00000100;		// set "BES" LED
	do {
		_delay_ms(50);
		while (keydown) {}	// first wait for key release
		while (!keydown) {}	// then wait for another keypress
			switch(ColRow) {
				case 0x510: Arg *= 10; Arg += 7; i++; break;
				case 0x610: Arg *= 10; Arg += 8; i++; break;
				case 0x710: Arg *= 10; Arg += 9; i++; break;
				case 0x520: Arg *= 10; Arg += 4; i++; break;
				case 0x620: Arg *= 10; Arg += 5; i++; break;
				case 0x720: Arg *= 10; Arg += 6; i++; break;
				case 0x540: Arg *= 10; Arg += 1; i++; break;
				case 0x640: Arg *= 10; Arg += 2; i++; break;
				case 0x740: Arg *= 10; Arg += 3; i++; break;
				case 0x680: Arg *= 10; Arg += 0; i++; break;
				default: if (ColRow > 0x8) i = 3; break;
			}			
	} while (i<3);
	PORTD &= 0b11111011;		// reset "BES" LED	
	return Arg;
}

uint8_t Confirm(void)		// show error state and wait for confirmation
{
uint8_t Arg;
uint8_t i = 0;
	PORTD = 0b00000100;		// set "BES" LED
	_delay_ms(50);
	while (keydown) {}	// first wait for key release
	do {
		_delay_ms(5);
		i++;
		if (i==50) PORTD = 0b00000000;
		if (i==100) {
			i = 0;
			PORTD = 0b00000100;
		}
	} while (!keydown);	// wait for a key
	if (ColRow == 0x508) Arg = 1;	// Key "L" is for Confirmation, every other key is "Abort Action"
	else Arg = 0;
	PORTD = 0b00000000;		// reset "BES" LED	
	return Arg;					// 0 means "Confirmation", -1 means "Abort Action"
}

uint8_t CheckBlock(uint16_t blkno)
{
uint8_t sum = 0;
uint16_t i;
	if (mmc_init()) hlt();
	mmc_read_sector((uint32_t)(blkno), KRam);	// read first 512 Bytes block
	for (i=0;i<512;i++) {
		sum |= KRam[i];
	}	
	return sum;
}